var BaseView = require('bassview');

var Trow = BaseView.extend({
    
    className: 'tr',
    
    initialize: function() {
        this.listenTo(this.model, "change change_selected", this.render);
    },
    
    render: function() {
        var html = '';
        this.collection.each(function(column){
            var id = column.get('id');
            var width = column.get('width');
            var formatted = column.getFormatted(this.model);
            html += '<div class="td col-' + id + '" style="width:' + width + 'px"><div class="cell-inner">' + formatted + '</div></div>';
        }, this);
        this.$el.html(html);
        return this;
    }
});



var Tbody = BaseView.extend({
    
    initialize: function(options) {
        this.columns = options.columns;
        this.config = this.columns.config;
        this.listenTo(this.collection, "reset sort sync tabled:update", this.render);
        // this.listenTo(this.collection, "update", this.render); // commented out because 'sort' event gets called anyway on update
        this.listenTo(this.config, "update change:offset", this.render);
        this.listenTo(this.columns, "change:width", this.adjustColumnWidth );
    },
    
    adjustColumnWidth: function(model, newWidth, options){
        this.$('.td.col-'+model.get("id")).width(newWidth);
    },
    
    // Renders the visible rows given the current offset and max_rows
    // properties on the config object.
    render: function() {
        this.$el.empty();
        this.trigger("clean_up");
        var rows_to_render = this.config.getVisibleRows();
        if (rows_to_render === false) return;
        
        rows_to_render.forEach(function(row){
            var rowView = new Trow({
                model: row,
                collection: this.columns
            });
            this.$el.append( rowView.render().el );
            rowView.listenTo(this, "clean_up", rowView.remove);
        }, this);
        this.trigger('rendered');
        return this;
    },
    
    events: {
        "wheel": "onMouseWheel",
        "mousewheel": "onMouseWheel"
    },
    
    onMouseWheel: function(evt) {
        // BigInteger
        var self = this;
        
        // normalize webkit/firefox scroll values
        var deltaY = -evt.originalEvent.wheelDeltaY || evt.originalEvent.deltaY * 100;
        var movement = Math.round(deltaY / 100);
        if (isNaN(movement)) return;
        var origOffset = this.config.get("offset");
        var limit = this.config.get("max_rows");
        var offset = Math.min( this.collection.length - limit, Math.max( 0, origOffset + movement));
        
        this.config.set({"offset": offset}, {validate: true} );
        
        // Prevent default only when necessary
        if (offset > 0 && offset < this.collection.length - limit) {
            evt.preventDefault();
            evt.originalEvent.preventDefault();
        }
    }
    
});
exports = module.exports = Tbody;